function works = will_agglomerative_work(data,labels)
% will_agglomerative_work(data,labels)
%    Returns true if simple-linkage agglomerative clustering
%    can perfectly cluster the columns of data, according to
%    labels.
%
%    This function is quite a bit faster than acutally performing
%    the clustering.
%
% Author: David Ross, Sept. 15, 2003.

classes = unique(labels);

% distances between points
% --- To use this function for other measures of distance,
% --- replace point_distance with your distance function.
% --- It should return a symmetric matrix containing the
% --- pairwise distances between all data points.
distances = point_distance(data);
distances = distances + diag(inf*ones(size(data,2),1));


%----------------------------------------------------------
% Intra-class distances.
%----------------------------------------------------------

% for each class
%   for each point in the class, compute the distance to it's 
%    nearest neighbour,
%    then find the maximum of this for points in the class
maxmin_distance = zeros(length(classes),1);
for cc = 1:length(classes)
    indicies = find(labels == classes(cc));
    maxmin_distance(cc) = max(min(distances,[],1));
end


%----------------------------------------------------------
% Inter-class distances.
%----------------------------------------------------------
% for each pair of classes, compute the smallest distance
% between a point in one class and a point in the other
interclass_distance = zeros(length(classes));
for c1 = 1:length(classes)
    for c2 = (c1+1):length(classes)
        ind1 = find(labels == classes(c1));
        ind2 = find(labels == classes(c2));
        interclass_distance(c1,c2) = ...
            min(min(distances(ind1,ind2)));
    end
end

interclass_distance = interclass_distance + interclass_distance' ...
    + diag(inf*ones(length(classes),1));

%----------------------------------------------------------
% Compare these distances.
%----------------------------------------------------------
works = 1; % true
for cc = 1:length(classes)
    if any(interclass_distance(cc,:) < maxmin_distance(cc))
        disp(sprintf('class %d won''t work', classes(cc)));
        works = false;
    end
end




%//////////////////////////////////////////////////////////
% d = point_distance(X)
%    Computes the pairwise distances between columns of X.
%----------------------------------------------------------
function d = point_distance(X)
N = size(X,2);
d = sum(X.^2,1);
d = ones(N,1)*d + d'*ones(1,N) - 2*X'*X;
